% This code implements a basic version of the persistence-based 
% segmentation algorithm, described in the paper:
% 
% Persistence-based Segmentation of Deformable Shapes,
% Primoz Skraba, Maks Ovsjanikov, Frederic Chazal and Leonidas Guibas, 
% Proc. NORDIA 2010
% 
% The code uses the basic persistence-based method (without randomization)
% as described in the paper, and produces results based on two functions:
% the HKS for a fixed t, and the WKS for a fixed energy value.
%
% The persistence computation itself is implemented in C++ and you might 
% need to run mex_compile.m to compile that part of the code.
%
% The code was written by Maks Ovsjanikov (maks@lix.polytechnique.fr)

addpath('./utils/');
addpath('./Mesh/');

fprintf('reading the shape...');
S1 = read_off_shape('mesh004.off');
C1 = [S1.surface.X S1.surface.Y S1.surface.Z];
fprintf('done\n');

fprintf('computing the Laplacian...');
A = vertexAreas(C1,S1.surface.TRIV);
W = cotWeights(C1,S1.surface.TRIV);
A = A / sum(A);

nv = S1.surface.nv;
% Compute the eigenvalues/vectors of the Laplacian.
[e v] = eigs(W,spdiags(A,0,nv,nv),30,-1e-6);

L1.A = spdiags(A,0,nv,nv);
L1.evecs = e;
L1.evals = diag(v);
fprintf('done\n');

fprintf('Segmentation with 2 functions...');
subplot(1,2,1);
f = hks(L1,0.01);
[S,P,I] = pb_clustering(S1.surface, f, 5);
plot_function_scape(S1.surface, full(S));
title('Segmentation based on the HKS');
hold on;

subplot(1,2,2);
f = wks(L1,5);
% parameters to clustering: surface, function, number of clusters
S = pb_clustering(S1.surface, f, 5);
plot_function_scape(S1.surface, full(S))
title('Segmentation based on the WKS');
fprintf('done\n');