% This code implements an extended version of the persistence-based 
% segmentation algorithm, described in the paper:
% 
% Persistence-based Segmentation of Deformable Shapes,
% Primoz Skraba, Maks Ovsjanikov, Frederic Chazal and Leonidas Guibas, 
% Proc. NORDIA 2010
% 
% The file uses the basic extended method (with randomization)
% as described in the paper, and produces results based on a single 
% functions: the HKS for a fixed t.
%
% The persistence computation itself is implemented in C++ and you might 
% need to run mex_compile.m to compile that part of the code.
%
% The code was written by Maks Ovsjanikov (maks@lix.polytechnique.fr)

clear all;
close all;
addpath('./utils/');
addpath('./GCMex/');
addpath('./Mesh/');

meshname = 'mesh004.off';
S1 = read_off_shape(meshname);
C1 = [S1.surface.X S1.surface.Y S1.surface.Z];

num_segments = 5;

fprintf('computing the Laplacian...');
A = vertexAreas(C1,S1.surface.TRIV);
W = cotWeights(C1,S1.surface.TRIV);
A = A / sum(A);

nv = S1.surface.nv;
[e v] = eigs(W,spdiags(A,0,nv,nv),60,-1e-6);

L1.A = spdiags(A,0,nv,nv);
L1.evecs = e;
L1.evals = diag(v);
fprintf('done\n');

%%
f = hks(L1,0.01);

[Ir, Pr, Sr] = mesh_persistence(S1.surface, f);

[S,P,I] = pb_clustering(S1.surface, f, num_segments);

pbs = zeros(length(f), num_segments);

nex = 200;
fprintf('extended persistence segmentation with %d iterations...', nex);
for m = 1:nex
    damp = 1/3;
    while(true)
        % perturb the function and record how the persistence association
        % changes.
        g = f+(rand(size(f))-1/2)*(max(f)-min(f))*damp;
        % parameters to clustering: surface, function, number of clusters
        [S2, P2, I2] = pb_clustering(S1.surface, g, num_segments);
        map(S2(I)) = 1:length(P);
        S3 = map(S2);
        if(min(S3)==0)
            damp = damp/2;
        else
            pbs = pbs + full(sparse(1:size(pbs,1),S3,1,size(pbs,1),size(pbs,2)));
            break;            
        end        
    end
end
fprintf('done\n');

%%
fprintf('post-processing with graph cuts...');
maxp = max(pbs,[],2)/nex;
Sf = S;
Sf(maxp < 0.9) = 0;

damp = 1.75;
while(true)    
    UN = [damp*nex-max(pbs,[],2) pbs];
    LC = 100*ones(num_segments+1, num_segments+1)-99*eye(num_segments+1, num_segments+1);
    l =  GCMex(Sf, -single(UN'), -W, single(LC));
    if(length(unique(l))<num_segments+1 && damp>1)
        damp = damp*0.9;
    else
        break;
    end
end
fprintf('done\n');

%%
plot_function_scape(S1.surface, l);
view([-120 25]);
title('Extended segmentation with the HKS');
